/**
 * Copyright (C) 2011
 *   Michael Mosmann <michael@mosmann.de>
 *   Martin Jöhren <m.joehren@googlemail.com>
 *
 * with contributions from
 * 	konstantin-ba@github,Archimedes Trajano (trajano@github)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.flapdoodle.embed.process.store;

import java.io.File;
import java.io.IOException;
import java.util.logging.Logger;

import de.flapdoodle.embed.process.config.store.IDownloadConfig;
import de.flapdoodle.embed.process.config.store.IPackageResolver;
import de.flapdoodle.embed.process.distribution.Distribution;
import de.flapdoodle.embed.process.extract.Extractors;
import de.flapdoodle.embed.process.extract.IExtractor;
import de.flapdoodle.embed.process.extract.ITempNaming;
import de.flapdoodle.embed.process.io.directories.IDirectory;
import de.flapdoodle.embed.process.io.file.Files;


public class ArtifactStore implements IArtifactStore {
	private static Logger logger = Logger.getLogger(ArtifactStore.class.getName());

	private IDownloadConfig _downloadConfig;
	private IDirectory _tempDireFactory;
	private ITempNaming _executableNaming;
	
	public ArtifactStore(IDownloadConfig downloadConfig,IDirectory tempDireFactory,ITempNaming executableNaming) {
		_downloadConfig=downloadConfig;
		_tempDireFactory = tempDireFactory;
		_executableNaming = executableNaming;
	}
	
	@Override
	public boolean checkDistribution(Distribution distribution) throws IOException {
		if (!LocalArtifactStore.checkArtifact(_downloadConfig, distribution)) {
			return LocalArtifactStore.store(_downloadConfig, distribution, Downloader.download(_downloadConfig, distribution));
		}
		return true;
	}

	@Override
	public File extractExe(Distribution distribution) throws IOException {
		IPackageResolver packageResolver = _downloadConfig.getPackageResolver();
		File artifact = LocalArtifactStore.getArtifact(_downloadConfig, distribution);
		IExtractor extractor = Extractors.getExtractor(packageResolver.getArchiveType(distribution));

		File exe = Files.createTempFile(_tempDireFactory,
				_executableNaming.nameFor("extract", packageResolver.executableFilename(distribution)));
		extractor.extract(_downloadConfig, artifact, exe, packageResolver.executeablePattern(distribution));
		return exe;
	}

	@Override
	public void removeExecutable(Distribution distribution, File executable) {
		if (executable.exists() && !Files.forceDelete(executable))
			logger.warning("Could not delete executable NOW: " + executable);
	}
}
